package com.aionnect.helloJava;

import com.fasterxml.jackson.databind.ObjectMapper;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;

/**
 * 从请求中解析用户名密码，经过userDetailsService验证正确后，生成一个token，并将token返回给客户端
 */
public class JWTLoginFilter extends UsernamePasswordAuthenticationFilter {
    private final AuthenticationManager authenticationManager;

    public JWTLoginFilter(AuthenticationManager authenticationManager) {
        this.authenticationManager = authenticationManager;
    }

    static class JWTUser {
        private String username;
        private String password;

        public String getUsername() {
            return username;
        }

        public void setUsername(String username) {
            this.username = username;
        }

        public String getPassword() {
            return password;
        }

        public void setPassword(String password) {
            this.password = password;
        }
    }

    // 接收并解析用户凭证
    // 默认POST login会进入此方法
    @Override
    public Authentication attemptAuthentication(HttpServletRequest req, HttpServletResponse res)
            throws AuthenticationException {
        JWTUser user = null;
        try {
            user = new ObjectMapper()
                    .readValue(req.getInputStream(), JWTUser.class);
        } catch (Exception ignored) {}

        if (null == user || null == user.getUsername() || user.getUsername().isEmpty()) {
            String username = req.getParameter("username");
            String password = req.getParameter("password");
            if (null != username && !username.isEmpty()) {
                if (null == user) {
                    user = new JWTUser();
                }
                user.setUsername(username);
                if ((null == user.getPassword() || user.getPassword().isEmpty()) && !password.isEmpty()) {
                    user.setPassword(password);
                }
            }
        }
        if (null == user || user.getUsername().isEmpty()) {
            throw new RuntimeException("Username can not be empty");
        }

        return authenticationManager.authenticate(
                new UsernamePasswordAuthenticationToken(
                        user.getUsername(),
                        user.getPassword(),
                        new ArrayList<>())
        );
    }

    // 用户成功登录后被调用，生成token
    @Override
    protected void successfulAuthentication(HttpServletRequest req,
                                            HttpServletResponse res,
                                            FilterChain chain,
                                            Authentication auth) throws IOException, ServletException {

        org.springframework.security.core.userdetails.User user =
                (org.springframework.security.core.userdetails.User) auth.getPrincipal();
        String token = Jwts.builder()
                .setSubject(user.getUsername())
                .setExpiration(new Date(System.currentTimeMillis() + 60 * 60 * 24 * 1000))
                .signWith(SignatureAlgorithm.HS512, "*(#U%TUJF#32")
                .compact();
        res.addHeader("Authorization", "Bearer " + token);
    }
}
